/*
 * Copyright 2002-2009 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.springframework.flex.samples.contact;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.springframework.flex.remoting.RemotingDestination;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.jdbc.core.simple.SimpleJdbcTemplate;

/**
 * 
 * @author Christophe Coenraets
 * @author Jeremy Grelle
 */
@RemotingDestination("contactService")
public class JdbcContactDAO implements ContactDAO {

    private final SimpleJdbcTemplate template;

    private final ParameterizedRowMapper<Contact> rowMapper = new ParameterizedRowMapper<Contact>() {

        public Contact mapRow(ResultSet rs, int rowNum) throws SQLException {
            Contact contact = new Contact();
            contact.setName(rs.getString("name"));
            contact.setTwitterId(rs.getString("twitter_id"));
            return contact;
        }
    };

    public JdbcContactDAO(DataSource dataSource) {
        this.template = new SimpleJdbcTemplate(dataSource);
    }

    public List<Contact> findAll(String userId) {
    	return this.template.query("SELECT * FROM contact WHERE name <> ? ORDER BY name", this.rowMapper, userId);
    }

	public Contact findById(String userId) {
		return this.template.queryForObject("SELECT * FROM contact WHERE name = ?", this.rowMapper, userId);
	}

}
