package org.springframework.flex.samples.runtime.config;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.flex.messaging.MessageTemplate;
import org.springframework.flex.remoting.RemotingDestination;
import org.springframework.flex.samples.ConnectNotification;
import org.springframework.flex.samples.DisconnectNotification;
import org.springframework.flex.samples.contact.Contact;
import org.springframework.flex.samples.contact.ContactDAO;
import org.springframework.security.context.SecurityContextHolder;

import flex.messaging.MessageBroker;
import flex.messaging.services.MessageService;

@RemotingDestination
public abstract class InstantMessengerService {

	private MessageBroker broker;
	
	private ContactDAO contactDAO;
	
	@Autowired
	public void setBroker(MessageBroker broker) {
		this.broker = broker;
	}
	
	@Autowired
	public void setContactService(ContactDAO contactDAO) {
		this.contactDAO = contactDAO;
	}

	public Map<String, List<Contact>> connect() {
		String userId = SecurityContextHolder.getContext().getAuthentication().getName();
		MessageService service = (MessageService) this.broker.getServiceByType(MessageService.class.getName());
		if (service.getDestination(userId) == null) {
			doCreate();
			System.out.println("CREATED USER DESTINATION: "+userId);
		}
		
		List<Contact> contacts = contactDAO.findAll(userId);
		List<Contact> onlineContacts = new ArrayList<Contact>();
		List<Contact> offlineContacts = new ArrayList<Contact>();
		for(Contact contact : contacts) {
			if (service.getDestination(contact.getName()) != null) {
				onlineContacts.add(contact);
			} else {
				offlineContacts.add(contact);
			}
		}
		Map<String, List<Contact>> contactsStatus = new HashMap<String, List<Contact>>();
		contactsStatus.put("online", onlineContacts);
		contactsStatus.put("offline", offlineContacts);
		Contact contact = contactDAO.findById(userId);
		publishConnectNotification(contact);
		return contactsStatus;
	}
	
	private void publishConnectNotification(Contact contact) {
		MessageTemplate template = new MessageTemplate();
		template.setMessageBroker(this.broker);
		template.send("notifications", new ConnectNotification(contact));
	}
	
	private void publishDisconnectNotification(Contact contact) {
		MessageTemplate template = new MessageTemplate();
		template.setMessageBroker(this.broker);
		template.send("notifications", new DisconnectNotification(contact));
	}

	public void disconnectUser(String userId) {
		MessageService service = (MessageService) this.broker.getServiceByType(MessageService.class.getName());
		if (service.getDestination(userId) != null) {
			service.getDestination(userId).stop();
			service.removeDestination(userId);
			System.out.println("REMOVED USER DESTINATION: "+userId);
		}
		Contact contact = contactDAO.findById(userId);
		publishDisconnectNotification(contact);
	}
	
	protected abstract void doCreate();

}
